@@ -2,8 +2,9 @@ import asyncio
 import contextvars
 import inspect
 import logging
+import typing
 from collections import ChainMap
-from contextlib import AsyncExitStack, AbstractAsyncContextManager, asynccontextmanager
+from contextlib import AsyncExitStack, AbstractAsyncContextManager, asynccontextmanager, contextmanager
 from json import JSONDecodeError
 from types import FunctionType, CoroutineType
 from typing import List, Union, Any, Callable, Type, Optional, Dict, Sequence, Awaitable, Tuple
@@ -31,7 +32,6 @@ from starlette.routing import Match, request_response, compile_path
 import fastapi.params
 import aiojobs
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -50,6 +50,14 @@ except ImportError:
             return value
 
 
+try:
+    import sentry_sdk
+    from sentry_sdk.utils import transaction_from_function as sentry_transaction_from_function
+except ImportError:
+    sentry_sdk = None
+    sentry_transaction_from_function = None
+
+
 class Params(fastapi.params.Body):
     def __init__(
         self,
@@ -92,6 +100,7 @@ def component_name(name: str, module: str = None):
     """OpenAPI components must be unique by name"""
     def decorator(obj):
         obj.__name__ = name
+        obj.__qualname__ = name
         if module is not None:
             obj.__module__ = module  # see: pydantic.schema.get_long_model_name
         key = (obj.__name__, obj.__module__)
@@ -493,7 +502,8 @@ class JsonRpcContext:
         http_request: Request,
         background_tasks: BackgroundTasks,
         http_response: Response,
-        json_rpc_request_class: Type[JsonRpcRequest] = JsonRpcRequest
+        json_rpc_request_class: Type[JsonRpcRequest] = JsonRpcRequest,
+        method_route: typing.Optional['MethodRoute'] = None,
     ):
         self.entrypoint: Entrypoint = entrypoint
         self.raw_request: Any = raw_request
@@ -501,6 +511,7 @@ class JsonRpcContext:
         self.background_tasks: BackgroundTasks = background_tasks
         self.http_response: Response = http_response
         self.request_class: Type[JsonRpcRequest] = json_rpc_request_class
+        self.method_route: typing.Optional[MethodRoute] = method_route
         self._raw_response: Optional[dict] = None
         self.exception: Optional[Exception] = None
         self.is_unhandled_exception: bool = False
@@ -546,6 +557,8 @@ class JsonRpcContext:
     async def __aenter__(self):
         assert self.exit_stack is None
         self.exit_stack = await AsyncExitStack().__aenter__()
+        if sentry_sdk is not None:
+            self.exit_stack.enter_context(self._fix_sentry_scope())
         await self.exit_stack.enter_async_context(self._handle_exception(reraise=False))
         self.jsonrpc_context_token = _jsonrpc_context.set(self)
         return self
@@ -578,6 +591,23 @@ class JsonRpcContext:
         if self.exception is not None and self.is_unhandled_exception:
             logger.exception(str(self.exception), exc_info=self.exception)
 
+    def _make_sentry_event_processor(self):
+        def event_processor(event, _):
+            if self.method_route is not None:
+                event['transaction'] = sentry_transaction_from_function(self.method_route.func)
+            return event
+
+        return event_processor
+
+    @contextmanager
+    def _fix_sentry_scope(self):
+        hub = sentry_sdk.Hub.current
+        with sentry_sdk.Hub(hub) as hub:
+            with hub.configure_scope() as scope:
+                scope.clear_breadcrumbs()
+                scope.add_event_processor(self._make_sentry_event_processor())
+                yield
+
     async def enter_middlewares(self, middlewares: Sequence['JsonRpcMiddleware']):
         for mw in middlewares:
             cm = mw(self)
@@ -758,6 +788,7 @@ class MethodRoute(APIRoute):
     ) -> dict:
         async with JsonRpcContext(
             entrypoint=self.entrypoint,
+            method_route=self,
             raw_request=req,
             http_request=http_request,
             background_tasks=background_tasks,
@@ -932,7 +963,6 @@ class EntrypointRoute(APIRoute):
         self.common_dependencies = common_dependencies
         self.request_class = request_class
 
-
     async def solve_shared_dependencies(
         self,
         http_request: Request,
@@ -1119,6 +1149,7 @@ class EntrypointRoute(APIRoute):
             match, child_scope = route.matches(http_request_shadow.scope)
             if match == Match.FULL:
                 # http_request is a transport layer and it is common for all JSON-RPC requests in a batch
+                ctx.method_route = route
                 return await route.handle_req(
                     http_request_shadow, background_tasks, sub_response, ctx,
                     dependency_cache=dependency_cache,
